from openpyxl.chart.area_chart import AreaChart
from openpyxl.chart.bar_chart import BarChart
from openpyxl.chart.line_chart import LineChart
from openpyxl.chart.pie_chart import PieChart
from openpyxl.chart.scatter_chart import ScatterChart
from openpyxl.chart.title import title_maker
from openpyxl.chart.legend import Legend
from openpyxl.chart.series import Series, SeriesLabel
from openpyxl.chart.data_source import NumDataSource, AxDataSource, StrRef, NumRef

chart_cls_mapping = {
    'area': AreaChart,
    'bar': BarChart,
    'line': LineChart,
    'pie': PieChart,
    'scatter': ScatterChart,
}


def chart_factory(chart_type):
    return chart_cls_mapping[chart_type]()


def create_title(title_obj):
    if not title_obj or title_obj.get('hidden', False):
        return None
    text = title_obj.get('text', '')
    if not text:
        return None
    return title_maker(text)


def create_legend(legend_obj):
    if not legend_obj or legend_obj.get('hidden', False):
        return None
    return Legend(legendPos=legend_obj.get('position', 'r'))


def set_marker(ser, marker_obj):
    if not marker_obj:
        return
    ser.marker.symbol = marker_obj.get('symbol', 'circle')
    ser.marker.size = marker_obj.get('size', 5)


def set_series(ser, ser_type, ser_obj):
    ser.tx = SeriesLabel(strRef=StrRef(ser_obj.get('tx'))
                         ) if 'tx' in ser_obj else None
    ser.cat = AxDataSource(numRef=NumRef(
        f=ser_obj.get('cat'))) if 'cat' in ser_obj else None
    ser.val = NumDataSource(numRef=NumRef(
        f=ser_obj.get('val'))) if 'val' in ser_obj else None

    if ser_type == 'scatter':
        ser.xVal = ser.cat
        ser.yVal = ser.val
        ser.spPr.ln.noFill = True
        set_marker(ser, ser_obj.get('marker'))

    if ser_type == 'line':
        ser.smooth = ser_obj.get('smooth', False)
        set_marker(ser, ser_obj.get('marker'))


def analyze_series(series_obj):
    series_map = dict()
    idx = 0
    for ser_obj in series_obj:
        ser_type = ser_obj.get('type', 'bar')
        ser = Series(idx=idx, order=idx)
        set_series(ser, ser_type, ser_obj)

        if not ser_type in series_map:
            series_map[ser_type] = list()
        series_map[ser_type].append(ser)
        idx += 1

    chart = None
    for chart_type in series_map:
        subchart = chart_factory(chart_type)
        for ser in series_map[chart_type]:
            subchart.ser.append(ser)
        if chart is None:
            chart = subchart
        else:
            chart += subchart
    return chart


def add_axis(chart, x_axis_obj, y_axis_obj):
    if x_axis_obj is not None:
        chart.x_axis.title = create_title(x_axis_obj.get('title'))
    if y_axis_obj is not None:
        chart.y_axis.title = create_title(y_axis_obj.get('title'))


def create_chart(chart_obj):
    plot_area_json = chart_obj.get('plotArea')
    if not plot_area_json:
        return None

    series_json = plot_area_json.get('series')
    if not series_json:
        return None

    chart = analyze_series(series_json)
    if chart is None:
        return None

    chart_type = chart_obj.get('chartType', 'bar')
    if chart_type != 'pie':
        x_axis = plot_area_json.get('xAxis', None)
        y_axis = plot_area_json.get('yAxis', None)
        add_axis(chart, x_axis, y_axis)

    # bar direction
    if chart_type == 'bar':
        chart.type = 'bar' if chart_obj.get('barDir') == 'row' else 'col'
    # grouping
    if chart_type in ('bar', 'line', 'area'):
        grouping = chart_obj.get('grouping', 'standard')
        # bug#6370
        if grouping == '':
            grouping = 'standard'
        chart.grouping = grouping
        if chart_type == 'bar' and chart.grouping == 'stacked':
            chart.overlap = 100

    plot_blank_as = plot_area_json.get('blankAs')
    if plot_blank_as:
        chart.display_blanks = plot_blank_as

    chart.title = create_title(chart_obj.get('title'))
    chart.legend = create_legend(chart_obj.get('legend'))

    chart.visible_cells_only = True
    return chart
