<?php
/**
 * This file is part of PHPPresentation - A pure PHP library for reading and writing
 * presentations documents.
 *
 * PHPPresentation is free software distributed under the terms of the GNU Lesser
 * General Public License version 3 as published by the Free Software Foundation.
 *
 * For the full copyright and license information, please read the LICENSE
 * file that was distributed with this source code. For the full list of
 * contributors, visit https://github.com/PHPOffice/PHPPresentation/contributors.
 *
 * @link        https://github.com/PHPOffice/PHPPresentation
 * @copyright   2009-2015 PHPPresentation contributors
 * @license     http://www.gnu.org/licenses/lgpl.txt LGPL version 3
 */
namespace PhpOffice\PhpPresentation\Writer\PowerPoint2007;

use PhpOffice\Common\Drawing as CommonDrawing;
use PhpOffice\Common\Text;
use PhpOffice\Common\XMLWriter;
use PhpOffice\PhpPresentation\Shape\AbstractGraphic;
use PhpOffice\PhpPresentation\Shape\Chart as ShapeChart;
use PhpOffice\PhpPresentation\Shape\Comment;
use PhpOffice\PhpPresentation\Shape\Drawing\Gd as ShapeDrawingGd;
use PhpOffice\PhpPresentation\Shape\Drawing\File as ShapeDrawingFile;
use PhpOffice\PhpPresentation\Shape\Group;
use PhpOffice\PhpPresentation\Shape\Line;
use PhpOffice\PhpPresentation\Shape\Media;
use PhpOffice\PhpPresentation\Shape\Placeholder;
use PhpOffice\PhpPresentation\Shape\RichText;
use PhpOffice\PhpPresentation\Shape\RichText\BreakElement;
use PhpOffice\PhpPresentation\Shape\RichText\Paragraph;
use PhpOffice\PhpPresentation\Shape\RichText\Run;
use PhpOffice\PhpPresentation\Shape\RichText\TextElement;
use PhpOffice\PhpPresentation\Shape\Table as ShapeTable;
use PhpOffice\PhpPresentation\Slide;
use PhpOffice\PhpPresentation\Slide\Animation;
use PhpOffice\PhpPresentation\Slide\Note;
use PhpOffice\PhpPresentation\Style\Alignment;
use PhpOffice\PhpPresentation\Style\Bullet;
use PhpOffice\PhpPresentation\Style\Border;
use PhpOffice\PhpPresentation\Style\Color;
use PhpOffice\PhpPresentation\Style\Font;
use PhpOffice\PhpPresentation\Style\Shadow;
use PhpOffice\PhpPresentation\Slide\AbstractSlide as AbstractSlideAlias;
use PhpOffice\PhpPresentation\Writer\PowerPoint2007\PptSlideLayouts;

abstract class AbstractSlide extends AbstractDecoratorWriter
{
    const DEFAULT_TOP_INSET = 45720;
    const DEFAULT_RIGHT_INSET = 91440;
    const DEFAULT_BOTTOM_INSET = 45720;
    const DEFAULT_LEFT_INSET = 91440;

    private $relationTypeNs = array(
        'theme' => 'http://schemas.openxmlformats.org/officeDocument/2006/relationships/theme',
        'master' => 'http://schemas.openxmlformats.org/officeDocument/2006/relationships/slideMaster',
        'layout' => 'http://schemas.openxmlformats.org/officeDocument/2006/relationships/slideLayout',
        'slide' => 'http://schemas.openxmlformats.org/officeDocument/2006/relationships/slide',
        'note' => 'http://schemas.openxmlformats.org/officeDocument/2006/relationships/notesSlide',
        'notesMaster' => 'http://schemas.openxmlformats.org/officeDocument/2006/relationships/notesMaster',
        'image' => 'http://schemas.openxmlformats.org/officeDocument/2006/relationships/image',
        'hyperlink' => 'http://schemas.openxmlformats.org/officeDocument/2006/relationships/hyperlink',
        'chart' => 'http://schemas.openxmlformats.org/officeDocument/2006/relationships/chart',
        'comment' => 'http://schemas.openxmlformats.org/officeDocument/2006/relationships/comments'
    );
    private $relationId = 1;
    private $relations = array();
    protected $shapeId = 1;
    protected $shapeIdMap = array();

    protected function resetRelationship()
    {
        $this->relationId = 1;
        $this->relations = array();
    }

    protected function getRelationId()
    {
        $rId = 'rId' . $this->relationId;
        $this->relationId++;

        return $rId;
    }

    protected function addRelation($rId, $type, $target)
    {
        if (!isset($rId) || !isset($type) || !isset($target)) {
            return;
        }
        $typeNs = $this->relationTypeNs[$type];
        if (!isset($typeNs)) {
            return;
        }

        $relation = array(
            'Id' => $rId,
            'Type' => $typeNs,
            'Target' => $target
        );
        if ('hyperlink' === $type) {
            $relation['TargetMode'] = 'External';
        }
        $this->relations[] = $relation;
    }

    protected function writeRelationships()
    {
        $objWriter = new XMLWriter(XMLWriter::STORAGE_MEMORY);
        $objWriter->startDocument('1.0', 'UTF-8', 'yes');
        $objWriter->startElement('Relationships');
        $objWriter->writeAttribute('xmlns', 'http://schemas.openxmlformats.org/package/2006/relationships');

        foreach ($this->relations as $relation) {
            $objWriter->startElement('Relationship');
            foreach ($relation as $key => $value) {
                $objWriter->writeAttribute($key, $value);
            }
            $objWriter->endElement();
        }
        $objWriter->endElement();

        return $objWriter->getData();
    }

    /**
     * @param XMLWriter $objWriter
     * @param \ArrayObject|\PhpOffice\PhpPresentation\AbstractShape[] $shapes
     * @throws \Exception
     */
    protected function writeShapeCollection(XMLWriter $objWriter, $shapes = array())
    {
        if (count($shapes) == 0) {
            return;
        }
        foreach ($shapes as $shape) {
            if ($shape instanceof RichText) {
                $this->writeShapeText($objWriter, $shape);
            } elseif ($shape instanceof ShapeTable) {
                $this->writeShapeTable($objWriter, $shape);
            } elseif ($shape instanceof Line) {
                $this->writeShapeLine($objWriter, $shape);
            } elseif ($shape instanceof ShapeChart) {
                $this->writeShapeChart($objWriter, $shape);
            } elseif ($shape instanceof AbstractGraphic) {
                $this->writeShapePic($objWriter, $shape);
            } elseif ($shape instanceof Group) {
                $this->writeShapeGroup($objWriter, $shape);
            } elseif ($shape instanceof Comment) {
                $this->hasComment = true;
            } else {
                throw new \Exception("Unknown Shape type: {get_class($shape)}");
            }
        }
    }

    /**
     * Write txt
     *
     * @param  \PhpOffice\Common\XMLWriter $objWriter XML Writer
     * @param  \PhpOffice\PhpPresentation\Shape\RichText $shape
     * @throws \Exception
     */
    protected function writeShapeText(XMLWriter $objWriter, RichText $shape)
    {
        // p:sp
        $objWriter->startElement('p:sp');
        // p:sp\p:nvSpPr
        $objWriter->startElement('p:nvSpPr');
        $this->writeCNvPr($objWriter, $shape);
        // p:sp\p:cNvSpPr
        $objWriter->startElement('p:cNvSpPr');
        if ($shape->isTextBox()) {
            $objWriter->writeAttribute('txBox', '1');
        }
        if ($shape->isPlaceholder()) {
            $objWriter->startElement('a:spLocks');
            $objWriter->writeAttribute('noGrp', '1');
            $objWriter->endElement();
        }
        $objWriter->endElement();
        $this->writeNvPr($objWriter, $shape);
        // > p:sp\p:cNvSpPr
        $objWriter->endElement();
        // p:sp\p:spPr
        $objWriter->startElement('p:spPr');

        $this->writeTransform($objWriter, $shape);
        // p:sp\p:spPr\a:prstGeom
        $objWriter->startElement('a:prstGeom');
        $objWriter->writeAttribute('prst', $shape->getShapeType());

        // p:sp\p:spPr\a:prstGeom\a:avLst
        $objWriter->startElement('a:avLst');
        $custPoints = $shape->getCustPoints();
        if (!is_null($custPoints)) {
            foreach ($custPoints as $name => $value) {
                $value = round(floatval($value));
                $objWriter->startElement('a:gd');
                $objWriter->writeAttribute('name', $name);
                $objWriter->writeAttribute('fmla', "val $value");
                $objWriter->endElement();
            }
        }
        $objWriter->endElement();

        // > p:sp\p:spPr
        $objWriter->endElement();

        $this->writeFill($objWriter, $shape->getFill());
        $this->writeBorder($objWriter, $shape->getBorder(), '');
        $this->writeShadow($objWriter, $shape->getShadow());

        // > p:sp\p:spPr
        $objWriter->endElement();
        // p:txBody
        $objWriter->startElement('p:txBody');
        // a:bodyPr
        //@link :http://msdn.microsoft.com/en-us/library/documentformat.openxml.drawing.bodyproperties%28v=office.14%29.aspx
        $objWriter->startElement('a:bodyPr');

        if ($shape->getWrap() != RichText::WRAP_SQUARE) {
            $objWriter->writeAttribute('wrap', $shape->getWrap());
        }
        if ($shape->getHorizontalOverflow() != RichText::OVERFLOW_OVERFLOW) {
            $objWriter->writeAttribute('horzOverflow', $shape->getHorizontalOverflow());
        }
        if ($shape->getVerticalOverflow() != RichText::OVERFLOW_OVERFLOW) {
            $objWriter->writeAttribute('vertOverflow', $shape->getVerticalOverflow());
        }
        if ($shape->isUpright()) {
            $objWriter->writeAttribute('upright', '1');
        }
        if ($shape->isVertical()) {
            $objWriter->writeAttribute('vert', 'vert');
        }
        $objWriter->writeAttribute('lIns', RichText::INSET_NONE !== $shape->getInsetLeft() ? CommonDrawing::pixelsToEmu($shape->getInsetLeft()) : self::DEFAULT_LEFT_INSET);
        $objWriter->writeAttribute('rIns', RichText::INSET_NONE !== $shape->getInsetRight() ? CommonDrawing::pixelsToEmu($shape->getInsetRight()) : self::DEFAULT_RIGHT_INSET);
        $objWriter->writeAttribute('tIns', RichText::INSET_NONE !== $shape->getInsetTop() ? CommonDrawing::pixelsToEmu($shape->getInsetTop()) : self::DEFAULT_TOP_INSET);
        $objWriter->writeAttribute('bIns', RichText::INSET_NONE !== $shape->getInsetBottom() ? CommonDrawing::pixelsToEmu($shape->getInsetBottom()) : self::DEFAULT_BOTTOM_INSET);
        $objWriter->writeAttribute('rtlCol', '0');
        if ('' !== ($verticalAlign = $shape->getAlignment()->getVertical())) {
            $objWriter->writeAttribute('anchor', $verticalAlign);
        }
        if ($shape->getColumns() <> 1) {
            $objWriter->writeAttribute('numCol', $shape->getColumns());
        }
        // a:spAutoFit
        $objWriter->startElement('a:' . $shape->getAutoFit());
        if ($shape->getAutoFit() == RichText::AUTOFIT_NORMAL) {
            if (!is_null($shape->getFontScale())) {
                $objWriter->writeAttribute('fontScale', (int)($shape->getFontScale() * 1000));
            }
            if (!is_null($shape->getLineSpaceReduction())) {
                $objWriter->writeAttribute('lnSpcReduction', (int)($shape->getLineSpaceReduction() * 1000));
            }
        }
        // > a:spAutoFit
        $objWriter->endElement();
        // > a:bodyPr
        $objWriter->endElement();

        // a:lstStyle
        if ($this instanceof PptSlideLayouts || (
            $shape->isPlaceholder() && $shape->getPlaceholder()->getType() === Placeholder::PH_TYPE_SLIDENUM
        )) {
            // a:lstStyle
            $objWriter->startElement('a:lstStyle');

            $paragraphs = $shape->getParagraphs();
            foreach ($paragraphs as $paragraph) {
                $align = $paragraph->getAlignment();
                $level = $align->getLevel();
                $bullet = $paragraph->getBulletStyle();

                $objWriter->startElement('a:lvl' . ($level + 1)  . 'pPr');
                if (Alignment::HORIZONTAL_NONE !== $align->getHorizontal()) {
                    $objWriter->writeAttribute('algn', $align->getHorizontal());
                }
                if (Bullet::TYPE_NONE === $bullet->getBulletType()) {
                    $objWriter->writeElement('a:buNone');
                }

                if (Paragraph::LINE_SPACE_NONE !== $paragraph->getLineSpacing()) {
                    $objWriter->startElement('a:lnSpc');
                    $objWriter->startElement('a:spcPct');
                    $objWriter->writeAttribute('val', $paragraph->getLineSpacing());
                    $objWriter->endElement();
                    $objWriter->endElement();
                }

                if (Paragraph::SPACE_BEF_AFT_NONE !== $paragraph->getSpaceBefore()) {
                    $objWriter->startElement('a:spcBef');
                    $objWriter->startElement('a:spcPts');
                    $objWriter->writeAttribute('val', $paragraph->getSpaceBefore());
                    $objWriter->endElement();
                    $objWriter->endElement();
                }

                if (Paragraph::SPACE_BEF_AFT_NONE !== $paragraph->getSpaceAfter()) {
                    $objWriter->startElement('a:spcAft');
                    $objWriter->startElement('a:spcPts');
                    $objWriter->writeAttribute('val', $paragraph->getSpaceAfter());
                    $objWriter->endElement();
                    $objWriter->endElement();
                }

                $objWriter->startElement('a:defRPr');
                $this->writeParagraphProperty($objWriter, $paragraph->getFont());
                $objWriter->endElement();

                $objWriter->endElement();
            }

            // > a:lstStyle
            $objWriter->endElement();
        }

        if ($shape->isPlaceholder() &&
            ($shape->getPlaceholder()->getType() == Placeholder::PH_TYPE_SLIDENUM ||
                $shape->getPlaceholder()->getType() == Placeholder::PH_TYPE_DATETIME)
        ) {
            $objWriter->startElement('a:p');
            $objWriter->startElement('a:fld');
            $objWriter->writeAttribute('id', $this->getGUID());
            $objWriter->writeAttribute('type', (
            $shape->getPlaceholder()->getType() == Placeholder::PH_TYPE_SLIDENUM ? 'slidenum' : 'datetime'));
            if ($shape->getPlaceholder()->getType() == Placeholder::PH_TYPE_SLIDENUM) {
                if ($this instanceof PptSlides) {
                    $objWriter->writeElement('a:t', $this->slideNumber);
                } else {
                    $objWriter->writeElement('a:t', '‹#›');
                }
            } else {
                $objWriter->writeElement('a:t', '03-04-05');
            }
            $objWriter->endElement();
            $objWriter->endElement();
        } else {
            // Write paragraphs
            $this->writeParagraphs($objWriter, $shape->getParagraphs(), !($this instanceof PptSlides) && $shape->isPlaceholder());
        }
        $objWriter->endElement();
        $objWriter->endElement();
    }

    /**
     * Write table
     *
     * @param  \PhpOffice\Common\XMLWriter $objWriter XML Writer
     * @param  \PhpOffice\PhpPresentation\Shape\Table $shape
     * @throws \Exception
     */
    protected function writeShapeTable(XMLWriter $objWriter, ShapeTable $shape)
    {
        // p:graphicFrame
        $objWriter->startElement('p:graphicFrame');
        // p:graphicFrame/p:nvGraphicFramePr
        $objWriter->startElement('p:nvGraphicFramePr');
        $this->writeCNvPr($objWriter, $shape);
        // p:graphicFrame/p:nvGraphicFramePr/p:cNvGraphicFramePr
        $objWriter->startElement('p:cNvGraphicFramePr');
        // p:graphicFrame/p:nvGraphicFramePr/p:cNvGraphicFramePr/a:graphicFrameLocks
        $objWriter->startElement('a:graphicFrameLocks');
        $objWriter->writeAttribute('noGrp', '1');
        $objWriter->endElement();
        // p:graphicFrame/p:nvGraphicFramePr/p:cNvGraphicFramePr/
        $objWriter->endElement();
        $this->writeNvPr($objWriter, $shape);
        // p:graphicFrame/p:nvGraphicFramePr/
        $objWriter->endElement();

        $this->writeTransform($objWriter, $shape);
        // p:graphicFrame/a:graphic
        $objWriter->startElement('a:graphic');
        // p:graphicFrame/a:graphic/a:graphicData
        $objWriter->startElement('a:graphicData');
        $objWriter->writeAttribute('uri', 'http://schemas.openxmlformats.org/drawingml/2006/table');
        // p:graphicFrame/a:graphic/a:graphicData/a:tbl
        $objWriter->startElement('a:tbl');
        // p:graphicFrame/a:graphic/a:graphicData/a:tbl/a:tblPr
        $objWriter->startElement('a:tblPr');
        $objWriter->writeAttribute('firstRow', '1');
        $objWriter->writeAttribute('bandRow', '1');
        $objWriter->endElement();
        // p:graphicFrame/a:graphic/a:graphicData/a:tbl/a:tblGrid
        $objWriter->startElement('a:tblGrid');
        // Write cell widths
        $countCells = count($shape->getRow(0)->getCells());
        for ($cell = 0; $cell < $countCells; $cell++) {
            //  p:graphicFrame/a:graphic/a:graphicData/a:tbl/a:tblGrid/a:gridCol
            $objWriter->startElement('a:gridCol');
            // Calculate column width
            $width = $shape->getRow(0)->getCell($cell)->getWidth();
            if ($width == 0) {
                $colCount = count($shape->getRow(0)->getCells());
                $totalWidth = $shape->getWidth();
                $width = $totalWidth / $colCount;
            }
            $objWriter->writeAttribute('w', CommonDrawing::pixelsToEmu($width));
            $objWriter->endElement();
        }
        // p:graphicFrame/a:graphic/a:graphicData/a:tbl/a:tblGrid/
        $objWriter->endElement();
        // Colspan / rowspan containers
        $colSpan = array();
        $rowSpan = array();
        // Default border style
        $defaultBorder = new Border();
        // Write rows
        $countRows = count($shape->getRows());
        for ($row = 0; $row < $countRows; $row++) {
            // p:graphicFrame/a:graphic/a:graphicData/a:tbl/a:tr
            $objWriter->startElement('a:tr');
            $objWriter->writeAttribute('h', CommonDrawing::pixelsToEmu($shape->getRow($row)->getHeight()));
            // Write cells
            $countCells = count($shape->getRow($row)->getCells());
            for ($cell = 0; $cell < $countCells; $cell++) {
                // Current cell
                $currentCell = $shape->getRow($row)->getCell($cell);
                // Next cell right
                $nextCellRight = $shape->getRow($row)->getCell($cell + 1, true);
                // Next cell below
                $nextRowBelow = $shape->getRow($row + 1, true);
                $nextCellBelow = null;
                if ($nextRowBelow != null) {
                    $nextCellBelow = $nextRowBelow->getCell($cell, true);
                }
                // a:tc
                $objWriter->startElement('a:tc');
                // Colspan
                if ($currentCell->getColSpan() > 1) {
                    $objWriter->writeAttribute('gridSpan', $currentCell->getColSpan());
                }
                if ($currentCell->isHMerge()) {
                    $objWriter->writeAttribute('hMerge', '1');
                }
                // Rowspan
                if ($currentCell->getRowSpan() > 1) {
                    $objWriter->writeAttribute('rowSpan', $currentCell->getRowSpan());
                }
                if ($currentCell->isVMerge()) {
                    $objWriter->writeAttribute('vMerge', '1');
                }
                // a:txBody
                $objWriter->startElement('a:txBody');
                // a:txBody/a:bodyPr
                $objWriter->startElement('a:bodyPr');
                //$objWriter->writeAttribute('wrap', 'square');
                //$objWriter->writeAttribute('rtlCol', '0');
                // a:txBody/a:bodyPr/a:spAutoFit
                //$objWriter->writeElement('a:spAutoFit', null);
                // a:txBody/a:bodyPr/
                $objWriter->endElement();
                // a:lstStyle
                $objWriter->writeElement('a:lstStyle', null);
                // Write paragraphs
                $this->writeParagraphs($objWriter, $currentCell->getParagraphs(), !($this instanceof PptSlides) && $shape->isPlaceholder());
                $objWriter->endElement();
                // a:tcPr
                $objWriter->startElement('a:tcPr');
                //FIXME
                $objWriter->writeAttribute('marL', '57600');
                $objWriter->writeAttribute('marR', '57600');
                if (Alignment::VERTICAL_NONE !== $currentCell->getAlignment()->getVertical()) {
                    switch ($currentCell->getAlignment()->getVertical()) {
                        case Alignment::VERTICAL_TOP:
                            $objWriter->writeAttribute('anchor', 't');
                            break;
                        case Alignment::VERTICAL_BOTTOM:
                            $objWriter->writeAttribute('anchor', 'b');
                            break;
                        default:
                            $objWriter->writeAttribute('anchor', 'ctr');
                            break;
                    }
                }
                $firstParagraph = $currentCell->getParagraph(0);
                $firstParagraphAlignment = $firstParagraph->getAlignment();

                // Text Direction
                $textDirection = $firstParagraphAlignment->getTextDirection();
                if ($textDirection != Alignment::TEXT_DIRECTION_HORIZONTAL) {
                    $objWriter->writeAttribute('vert', $textDirection);
                }
                // Alignment (horizontal)
                $verticalAlign = $firstParagraphAlignment->getVertical();
                if ($verticalAlign != Alignment::VERTICAL_BASE && $verticalAlign != Alignment::VERTICAL_AUTO && $verticalAlign != Alignment::VERTICAL_NONE) {
                    $objWriter->writeAttribute('anchor', $verticalAlign);
                }

                // Margins
                if (Alignment::MARGIN_NONE !== $firstParagraphAlignment->getMarginLeft()) {
                    $objWriter->writeAttribute('marL', CommonDrawing::pixelsToEmu($firstParagraphAlignment->getMarginLeft()));
                }
                if (Alignment::MARGIN_NONE !== $firstParagraphAlignment->getMarginRight()) {
                    $objWriter->writeAttribute('marR', CommonDrawing::pixelsToEmu($firstParagraphAlignment->getMarginRight()));
                }
                if (Alignment::MARGIN_NONE !== $firstParagraphAlignment->getMarginTop()) {
                    $objWriter->writeAttribute('marT', CommonDrawing::pixelsToEmu($firstParagraphAlignment->getMarginTop()));
                }
                if (Alignment::MARGIN_NONE !== $firstParagraphAlignment->getMarginBottom()) {
                    $objWriter->writeAttribute('marB', CommonDrawing::pixelsToEmu($firstParagraphAlignment->getMarginBottom()));
                }

                // Determine borders
                $borderLeft = $currentCell->getBorders()->getLeft();
                $borderRight = $currentCell->getBorders()->getRight();
                $borderTop = $currentCell->getBorders()->getTop();
                $borderBottom = $currentCell->getBorders()->getBottom();
                $borderDiagonalDown = $currentCell->getBorders()->getDiagonalDown();
                $borderDiagonalUp = $currentCell->getBorders()->getDiagonalUp();
                // Fix PowerPoint implementation
                if (!is_null($nextCellRight)
                    && $nextCellRight->getBorders()->getRight()->getHashCode() != $defaultBorder->getHashCode()
                ) {
                    $borderRight = $nextCellRight->getBorders()->getLeft();
                }
                if (!is_null($nextCellBelow)
                    && $nextCellBelow->getBorders()->getBottom()->getHashCode() != $defaultBorder->getHashCode()
                ) {
                    $borderBottom = $nextCellBelow->getBorders()->getTop();
                }
                // Write borders
                $this->writeBorder($objWriter, $borderLeft, 'L');
                $this->writeBorder($objWriter, $borderRight, 'R');
                $this->writeBorder($objWriter, $borderTop, 'T');
                $this->writeBorder($objWriter, $borderBottom, 'B');
                $this->writeBorder($objWriter, $borderDiagonalDown, 'TlToBr');
                $this->writeBorder($objWriter, $borderDiagonalUp, 'BlToTr');
                // Fill
                $this->writeFill($objWriter, $currentCell->getFill());
                $objWriter->endElement();
                $objWriter->endElement();
            }
            $objWriter->endElement();
        }
        $objWriter->endElement();
        $objWriter->endElement();
        $objWriter->endElement();
        $objWriter->endElement();
    }

    /**
     * Write paragraphs
     *
     * @param  \PhpOffice\Common\XMLWriter $objWriter XML Writer
     * @param  \PhpOffice\PhpPresentation\Shape\RichText\Paragraph[] $paragraphs
     * @param  bool $bIsPlaceholder
     * @throws \Exception
     */
    protected function writeParagraphs(XMLWriter $objWriter, $paragraphs, $bIsPlaceholder)
    {
        if (0 === count($paragraphs)) {
            $objWriter->startElement('a:p');
            $objWriter->writeElement('a:endParaRPr');
            $objWriter->endElement();
            return;
        }
        // Loop trough paragraphs
        foreach ($paragraphs as $paragraph) {
            $align = $paragraph->getAlignment();
            $bulletStyle = $paragraph->getBulletStyle();
            $bulletType = $bulletStyle->getBulletType();

            // a:p
            $objWriter->startElement('a:p');

            // a:pPr
            $objWriter->startElement('a:pPr');
            //if (Bullet::TYPE_NONE !== $bulletType && Bullet::TYPE_NOTSET !== $bulletType) {
            //if (Bullet::TYPE_NONE !== $bulletType) {
            if (0 !== $align->getLevel()) {
                $objWriter->writeAttribute('lvl', $align->getLevel());
            }

            if (!$bIsPlaceholder) {
                if (Alignment::HORIZONTAL_NONE !== $align->getHorizontal()) {
                    $objWriter->writeAttribute('algn', $align->getHorizontal());
                }
                if (Alignment::VERTICAL_NONE !== $align->getVertical()) {
                    $objWriter->writeAttribute('fontAlgn', $align->getVertical());
                }
                if (Alignment::MARGIN_NONE !== $align->getMarginLeft()) {
                    $objWriter->writeAttribute('marL', CommonDrawing::pixelsToEmu($align->getMarginLeft()));
                }
                if (Alignment::MARGIN_NONE !== $align->getMarginTop()) {
                    $objWriter->writeAttribute('marT', CommonDrawing::pixelsToEmu($align->getMarginTop()));
                }
                if (Alignment::MARGIN_NONE !== $align->getMarginRight()) {
                    $objWriter->writeAttribute('marR', CommonDrawing::pixelsToEmu($align->getMarginRight()));
                }
                if (Alignment::MARGIN_NONE !== $align->getMarginBottom()) {
                    $objWriter->writeAttribute('marB', CommonDrawing::pixelsToEmu($align->getMarginBottom()));
                }
                if (Alignment::INDENT_NONE !== $align->getIndent()) {
                    $objWriter->writeAttribute('indent', CommonDrawing::pixelsToEmu($align->getIndent()));
                }

                if (Paragraph::LINE_SPACE_NONE !== $paragraph->getLineSpacing()) {
                    $objWriter->startElement('a:lnSpc');
                    $objWriter->startElement('a:spcPct');
                    $objWriter->writeAttribute('val', $paragraph->getLineSpacing());
                    $objWriter->endElement();
                    $objWriter->endElement();
                }

                if (Paragraph::SPACE_BEF_AFT_NONE !== $paragraph->getSpaceBefore()) {
                    $objWriter->startElement('a:spcBef');
                    $objWriter->startElement('a:spcPts');
                    $objWriter->writeAttribute('val', $paragraph->getSpaceBefore());
                    $objWriter->endElement();
                    $objWriter->endElement();
                }

                if (Paragraph::SPACE_BEF_AFT_NONE !== $paragraph->getSpaceAfter()) {
                    $objWriter->startElement('a:spcAft');
                    $objWriter->startElement('a:spcPts');
                    $objWriter->writeAttribute('val', $paragraph->getSpaceAfter());
                    $objWriter->endElement();
                    $objWriter->endElement();
                }

                // Bullet type specified?
                if (Bullet::TYPE_NONE !== $bulletType && Bullet::TYPE_NOTSET !== $bulletType) {
                    // Color
                    // a:buClr must be before a:buFont (else PowerPoint crashes at launch)
                    if ($bulletStyle->getBulletColor() instanceof Color) {
                        $objWriter->startElement('a:buClr');
                        $this->writeColor($objWriter, $bulletStyle->getBulletColor());
                        $objWriter->endElement();
                    }

                    // a:buFont
                    $objWriter->startElement('a:buFont');
                    $objWriter->writeAttribute('typeface', $bulletStyle->getBulletFont());
                    $objWriter->endElement();

                    if (Bullet::TYPE_BULLET === $bulletType) {
                        // a:buChar
                        $objWriter->startElement('a:buChar');
                        $objWriter->writeAttribute('char', $bulletStyle->getBulletChar());
                        $objWriter->endElement();
                    } elseif (Bullet::TYPE_NUMERIC === $bulletType) {
                        // a:buAutoNum
                        $objWriter->startElement('a:buAutoNum');
                        $objWriter->writeAttribute('type', $bulletStyle->getBulletNumericStyle());
                        if ($bulletStyle->getBulletNumericStartAt() != 1) {
                            $objWriter->writeAttribute('startAt', $bulletStyle->getBulletNumericStartAt());
                        }
                        $objWriter->endElement();
                    }
                } else {
                    $objWriter->writeElement('a:buNone');
                }
                //$objWriter->endElement();
            }
            $objWriter->endElement();

            // Loop trough rich text elements
            $elements = $paragraph->getRichTextElements();
            foreach ($elements as $element) {
                if ($element instanceof BreakElement) {
                    // a:br
                    $objWriter->writeElement('a:br', null);
                } elseif ($element instanceof Run || $element instanceof TextElement) {
                    // a:r
                    $objWriter->startElement('a:r');

                    // a:rPr
                    if ($element instanceof Run && !$bIsPlaceholder) {
                        // a:rPr
                        $objWriter->startElement('a:rPr');

                        // Lang
                        $objWriter->writeAttribute('lang', ($element->getLanguage() ? $element->getLanguage() : 'en-US'));

                        $font = $element->getFont();
                        $this->writeParagraphProperty($objWriter, $font);

                        // Character spacing
                        $objWriter->writeAttribute('spc', $font->getCharacterSpacing());

                        // a:hlinkClick
                        $this->writeHyperlink($objWriter, $element);

                        $objWriter->endElement();
                    }

                    // t
                    $objWriter->startElement('a:t');
                    //$objWriter->writeCData(Text::controlCharacterPHP2OOXML($element->getText()));
                    $objWriter->text(Text::controlCharacterPHP2OOXML($element->getText()));
                    $objWriter->endElement();

                    $objWriter->endElement();
                }
            }

            $objWriter->writeElement('a:endParaRPr');
            $objWriter->endElement();
        }
    }

    /**
     * Write Line Shape
     *
     * @param  \PhpOffice\Common\XMLWriter $objWriter XML Writer
     * @param \PhpOffice\PhpPresentation\Shape\Line $shape
     */
    protected function writeShapeLine(XMLWriter $objWriter, Line $shape)
    {
        // p:sp
        $objWriter->startElement('p:cxnSp');
        // p:nvSpPr
        $objWriter->startElement('p:nvCxnSpPr');
        $this->writeCNvPr($objWriter, $shape);
        // p:cNvCxnSpPr
        $objWriter->writeElement('p:cNvCxnSpPr', null);
        $this->writeNvPr($objWriter, $shape);
        $objWriter->endElement();
        // p:spPr
        $objWriter->startElement('p:spPr');
        $this->writeTransform($objWriter, $shape);
        // a:prstGeom
        $objWriter->startElement('a:prstGeom');
        $objWriter->writeAttribute('prst', $shape->getShapeType());

        // a:prstGeom/a:avLst
        $objWriter->startElement('a:avLst');
        $custPoints = $shape->getCustPoints();
        if (!is_null($custPoints)) {
            foreach ($custPoints as $name => $value) {
                $value = round(floatval($value));
                $objWriter->startElement('a:gd');
                $objWriter->writeAttribute('name', $name);
                $objWriter->writeAttribute('fmla', "val $value");
                $objWriter->endElement();
            }
        }
        $objWriter->endElement();

        $objWriter->endElement();

        $lineEnd = '';
        if ($shape->getTailEnd() && $shape->getHeadEnd()) {
            $lineEnd = 'both';
        } else if ($shape->getTailEnd()) {
            $lineEnd = 'tail';
        } else if ($shape->getHeadEnd()) {
            $lineEnd = 'head';
        }

        $this->writeBorder($objWriter, $shape->getBorder(), '', $lineEnd);
        $objWriter->endElement();
        $objWriter->endElement();
    }

    /**
     * Write Shadow
     * @param XMLWriter $objWriter
     * @param Shadow $oShadow
     */
    protected function writeShadow(XMLWriter $objWriter, $oShadow)
    {
        if (!($oShadow instanceof Shadow)) {
            return;
        }

        if (!$oShadow->isVisible()) {
            return;
        }

        // a:effectLst
        $objWriter->startElement('a:effectLst');

        // a:outerShdw
        $objWriter->startElement('a:outerShdw');
        $objWriter->writeAttribute('blurRad', CommonDrawing::pixelsToEmu($oShadow->getBlurRadius()));
        $objWriter->writeAttribute('dist', CommonDrawing::pixelsToEmu($oShadow->getDistance()));
        $objWriter->writeAttribute('dir', CommonDrawing::degreesToAngle($oShadow->getDirection()));
        $objWriter->writeAttribute('algn', $oShadow->getAlignment());
        $objWriter->writeAttribute('rotWithShape', '0');

        $this->writeColor($objWriter, $oShadow->getColor(), $oShadow->getAlpha());

        $objWriter->endElement();

        $objWriter->endElement();
    }

    /**
     * Write hyperlink
     *
     * @param \PhpOffice\Common\XMLWriter $objWriter XML Writer
     * @param \PhpOffice\PhpPresentation\AbstractShape|\PhpOffice\PhpPresentation\Shape\RichText\TextElement $shape
     */
    protected function writeHyperlink(XMLWriter $objWriter, $shape)
    {
        if (!$shape->hasHyperlink()) {
            return;
        }
        $hyperlink = $shape->getHyperlink();

        $rId = $this->getRelationId();

        // a:hlinkClick
        $objWriter->startElement('a:hlinkClick');
        $objWriter->writeAttribute('r:id', $rId);
        $objWriter->writeAttribute('tooltip', $hyperlink->getTooltip());
        if ($hyperlink->isInternal()) {
            $objWriter->writeAttribute('action', $hyperlink->getAction());
            $this->addRelation($rId, 'slide', $hyperlink->getUrl());
        } else {
            $this->addRelation($rId, 'hyperlink', $hyperlink->getUrl());
        }
        $objWriter->endElement();
    }

    /**
     * Write chart
     *
     * @param \PhpOffice\Common\XMLWriter $objWriter XML Writer
     * @param \PhpOffice\PhpPresentation\Shape\Chart $shape
     */
    protected function writeShapeChart(XMLWriter $objWriter, ShapeChart $shape)
    {
        $rId = $this->getRelationId();
        $this->addRelation($rId, 'chart', '../charts/' . $shape->getIndexedFilename());

        // p:graphicFrame
        $objWriter->startElement('p:graphicFrame');
        // p:nvGraphicFramePr
        $objWriter->startElement('p:nvGraphicFramePr');
        $this->writeCNvPr($objWriter, $shape);
        // p:cNvGraphicFramePr
        $objWriter->writeElement('p:cNvGraphicFramePr', null);
        $this->writeNvPr($objWriter, $shape);
        $objWriter->endElement();
        $this->writeTransform($objWriter, $shape);
        // a:graphic
        $objWriter->startElement('a:graphic');
        // a:graphicData
        $objWriter->startElement('a:graphicData');
        $objWriter->writeAttribute('uri', 'http://schemas.openxmlformats.org/drawingml/2006/chart');
        // c:chart
        $objWriter->startElement('c:chart');
        $objWriter->writeAttribute('xmlns:c', 'http://schemas.openxmlformats.org/drawingml/2006/chart');
        $objWriter->writeAttribute('xmlns:r', 'http://schemas.openxmlformats.org/officeDocument/2006/relationships');
        $objWriter->writeAttribute('r:id', $rId);
        $objWriter->endElement();
        $objWriter->endElement();
        $objWriter->endElement();
        $objWriter->endElement();
    }

    /**
     * Write pic
     *
     * @param  \PhpOffice\Common\XMLWriter $objWriter XML Writer
     * @param  \PhpOffice\PhpPresentation\Shape\AbstractGraphic $shape
     * @throws \Exception
     */
    protected function writeShapePic(XMLWriter $objWriter, AbstractGraphic $shape)
    {
        $rId = $this->getRelationId();
        $this->addRelation($rId, 'image', '../media/' . $shape->getIndexedFilename());

        // p:pic
        $objWriter->startElement('p:pic');
        // p:nvPicPr
        $objWriter->startElement('p:nvPicPr');
        $this->writeCNvPr($objWriter, $shape);
        // p:cNvPicPr
        $objWriter->startElement('p:cNvPicPr');
        // a:picLocks
        $objWriter->startElement('a:picLocks');
        $objWriter->writeAttribute('noChangeAspect', '1');
        $objWriter->endElement();
        $objWriter->endElement();
        $this->writeNvPr($objWriter, $shape);
        $objWriter->endElement();
        // p:blipFill
        $objWriter->startElement('p:blipFill');
        // a:blip
        $objWriter->startElement('a:blip');
        $objWriter->writeAttribute('r:embed', $rId);
        // a:alphaModFix
        if (100 > $shape->getAlpha()) {
            $objWriter->startElement('a:alphaModFix');
            $objWriter->writeAttribute('amt', $shape->getAlpha() * 1000);
            // > a:alphaModFix
            $objWriter->endElement();
        }
        $objWriter->endElement();
        //a:srcRect
        list($this->cropTop, $this->cropRight, $this->cropBottom, $this->cropLeft) = $shape->getCrop();
        $objWriter->startElement('a:srcRect');
        $objWriter->writeAttribute('t', $this->cropTop);
        $objWriter->writeAttribute('r', $this->cropRight);
        $objWriter->writeAttribute('b', $this->cropBottom);
        $objWriter->writeAttribute('l', $this->cropLeft);
        $objWriter->endElement();
        // a:stretch
        $objWriter->startElement('a:stretch');
        $objWriter->writeElement('a:fillRect', null);
        $objWriter->endElement();
        $objWriter->endElement();
        // p:spPr
        $objWriter->startElement('p:spPr');
        $this->writeTransform($objWriter, $shape);
        // a:prstGeom
        $objWriter->startElement('a:prstGeom');
        $objWriter->writeAttribute('prst', $shape->getShapeType());
        // a:avLst
        $objWriter->writeElement('a:avLst', null);
        $objWriter->endElement();
        if ($shape->getBorder()->getLineStyle() != Border::LINE_NONE && $shape->getBorder()->getLineStyle() != Border::LINE_NOTSET) {
            $this->writeBorder($objWriter, $shape->getBorder(), '');
        }
        if ($shape->getShadow()->isVisible()) {
            $this->writeShadow($objWriter, $shape->getShadow());
        }
        $objWriter->endElement();
        $objWriter->endElement();
    }

    /**
     * Write group
     *
     * @param \PhpOffice\Common\XMLWriter $objWriter XML Writer
     * @param \PhpOffice\PhpPresentation\Shape\Group $group
     */
    protected function writeShapeGroup(XMLWriter $objWriter, Group $group)
    {
        // p:grpSp
        $objWriter->startElement('p:grpSp');
        // p:nvGrpSpPr
        $objWriter->startElement('p:nvGrpSpPr');
        // p:cNvPr
        $this->writeCNvPr($objWriter, $group);
        // p:cNvGrpSpPr
        $objWriter->writeElement('p:cNvGrpSpPr', null);
        $this->writeNvPr($objWriter, $group);
        $objWriter->endElement(); // p:nvGrpSpPr
        // p:grpSpPr
        $objWriter->startElement('p:grpSpPr');
        $this->writeTransform($objWriter, $group);
        $objWriter->endElement(); // p:grpSpPr

        $this->writeShapeCollection($objWriter, $group->getShapeCollection());

        $objWriter->endElement(); // p:grpSp
    }

    /**
     * @param \PhpOffice\PhpPresentation\Slide\AbstractSlide $pSlide
     * @param $objWriter
     */
    protected function writeSlideBackground(AbstractSlideAlias $pSlide, XMLWriter $objWriter)
    {
        $oBackground = $pSlide->getBackground();
        if (!($oBackground instanceof Slide\AbstractBackground)) {
            return;
        }

        // p:bg
        $objWriter->startElement('p:bg');
        if ($oBackground instanceof Slide\Background\Color) {
            $oColor = $oBackground->getColor();

            // p:bgPr
            $objWriter->startElement('p:bgPr');

            if (0 == $oColor->getAlpha()) {
                $objWriter->writeElement('a:noFill');
            } else {
                $objWriter->startElement('a:solidFill');
                $this->writeColor($objWriter, $oColor);
                $objWriter->endElement();
            }

            // p:bgPr/a:effectLst
            $objWriter->writeElement('a:effectLst');

            // > p:bgPr
            $objWriter->endElement();
        }
        if ($oBackground instanceof Slide\Background\Image) {
            $rId = 'rId' . $this->getRelationId();
            $this->addRelation($rId, 'image', '../media/' . $oBackground->getIndexedFilename());

            // p:bgPr
            $objWriter->startElement('p:bgPr');
            // a:blipFill
            $objWriter->startElement('a:blipFill');
            // a:blip
            $objWriter->startElement('a:blip');
            $objWriter->writeAttribute('r:embed', $rId);
            // a:alphaModFix
            if (100 > $oBackground->getAlpha()) {
                $objWriter->startElement('a:alphaModFix');
                $objWriter->writeAttribute('amt', $oBackground->getAlpha() * 1000);
                // > a:alphaModFix
                $objWriter->endElement();
            }
            // > a:blipFill
            $objWriter->endElement();
            // a:stretch
            $objWriter->startElement('a:stretch');
            // a:fillRect
            $objWriter->writeElement('a:fillRect');
            // > a:stretch
            $objWriter->endElement();
            // > a:blipFill
            $objWriter->endElement();
            // p:bgPr/a:effectLst
            $objWriter->writeElement('a:effectLst');
            // > p:bgPr
            $objWriter->endElement();
        }
        /**
         * @link : http://www.officeopenxml.com/prSlide-background.php
         */
        if ($oBackground instanceof Slide\Background\SchemeColor) {
            // p:bgRef
            $objWriter->startElement('p:bgRef');
            $objWriter->writeAttribute('idx', '1001');
            // a:schemeClr
            $objWriter->startElement('a:schemeClr');
            $objWriter->writeAttribute('val', $oBackground->getSchemeColor()->getValue());
            $objWriter->endElement();
            // > p:bgRef
            $objWriter->endElement();
        }
        // > p:bg
        $objWriter->endElement();
    }

    protected function writeTransition(XMLWriter $objWriter, $transition, $duration = false)
    {
        $objWriter->startElement('p:transition');
        $objWriter->writeAttribute('advClick', $transition->hasManualTrigger() ? '1' : '0');
        if ($transition->hasTimeTrigger()) {
            $objWriter->writeAttribute('advTm', $transition->getAdvanceTimeTrigger());
        }
        if (!is_null($transition->getSpeed())) {
            $objWriter->writeAttribute('spd', $transition->getSpeed());
        }
        if ($duration && !is_null($transition->getDuration())) {
            $objWriter->writeAttribute('p14:dur', $transition->getDuration());
        }

        switch ($transition->getTransitionType()) {
            case Slide\Transition::TRANSITION_BLINDS_HORIZONTAL:
                $objWriter->startElement('p:blinds');
                $objWriter->writeAttribute('dir', 'horz');
                $objWriter->endElement();
                break;
            case Slide\Transition::TRANSITION_BLINDS_VERTICAL:
                $objWriter->startElement('p:blinds');
                $objWriter->writeAttribute('dir', 'vert');
                $objWriter->endElement();
                break;
            case Slide\Transition::TRANSITION_CHECKER_HORIZONTAL:
                $objWriter->startElement('p:checker');
                $objWriter->writeAttribute('dir', 'horz');
                $objWriter->endElement();
                break;
            case Slide\Transition::TRANSITION_CHECKER_VERTICAL:
                $objWriter->startElement('p:checker');
                $objWriter->writeAttribute('dir', 'vert');
                $objWriter->endElement();
                break;
            case Slide\Transition::TRANSITION_CIRCLE:
                $objWriter->startElement('p:circle');
                $objWriter->endElement();
                break;
            case Slide\Transition::TRANSITION_COMB_HORIZONTAL:
                $objWriter->startElement('p:comb');
                $objWriter->writeAttribute('dir', 'horz');
                $objWriter->endElement();
                break;
            case Slide\Transition::TRANSITION_COMB_VERTICAL:
                $objWriter->startElement('p:comb');
                $objWriter->writeAttribute('dir', 'vert');
                $objWriter->endElement();
                break;
            case Slide\Transition::TRANSITION_COVER_DOWN:
                $objWriter->startElement('p:cover');
                $objWriter->writeAttribute('dir', 'd');
                $objWriter->endElement();
                break;
            case Slide\Transition::TRANSITION_COVER_LEFT:
                $objWriter->startElement('p:cover');
                $objWriter->writeAttribute('dir', 'l');
                $objWriter->endElement();
                break;
            case Slide\Transition::TRANSITION_COVER_LEFT_DOWN:
                $objWriter->startElement('p:cover');
                $objWriter->writeAttribute('dir', 'ld');
                $objWriter->endElement();
                break;
            case Slide\Transition::TRANSITION_COVER_LEFT_UP:
                $objWriter->startElement('p:cover');
                $objWriter->writeAttribute('dir', 'lu');
                $objWriter->endElement();
                break;
            case Slide\Transition::TRANSITION_COVER_RIGHT:
                $objWriter->startElement('p:cover');
                $objWriter->writeAttribute('dir', 'r');
                $objWriter->endElement();
                break;
            case Slide\Transition::TRANSITION_COVER_RIGHT_DOWN:
                $objWriter->startElement('p:cover');
                $objWriter->writeAttribute('dir', 'rd');
                $objWriter->endElement();
                break;
            case Slide\Transition::TRANSITION_COVER_RIGHT_UP:
                $objWriter->startElement('p:cover');
                $objWriter->writeAttribute('dir', 'ru');
                $objWriter->endElement();
                break;
            case Slide\Transition::TRANSITION_COVER_UP:
                $objWriter->startElement('p:cover');
                $objWriter->writeAttribute('dir', 'u');
                $objWriter->endElement();
                break;
            case Slide\Transition::TRANSITION_CUT:
                $objWriter->writeElement('p:cut');
                break;
            case Slide\Transition::TRANSITION_CUT_THROUGH_BLACK:
                $objWriter->startElement('p:cut');
                $objWriter->writeAttribute('thruBlk', '1');
                $objWriter->endElement();
                break;
            case Slide\Transition::TRANSITION_DIAMOND:
                $objWriter->writeElement('p:diamond');
                break;
            case Slide\Transition::TRANSITION_DISSOLVE:
                $objWriter->writeElement('p:dissolve');
                break;
            case Slide\Transition::TRANSITION_FADE:
                $objWriter->writeElement('p:fade');
                break;
            case Slide\Transition::TRANSITION_FADE_THROUGH_BLACK:
                $objWriter->startElement('p:fade');
                $objWriter->writeAttribute('thruBlk', '1');
                $objWriter->endElement();
                break;
            case Slide\Transition::TRANSITION_NEWSFLASH:
                $objWriter->writeElement('p:newsflash');
                break;
            case Slide\Transition::TRANSITION_PLUS:
                $objWriter->writeElement('p:plus');
                break;
            case Slide\Transition::TRANSITION_PULL_DOWN:
                $objWriter->startElement('p:pull');
                $objWriter->writeAttribute('dir', 'd');
                $objWriter->endElement();
                break;
            case Slide\Transition::TRANSITION_PULL_LEFT:
                $objWriter->startElement('p:pull');
                $objWriter->writeAttribute('dir', 'l');
                $objWriter->endElement();
                break;
            case Slide\Transition::TRANSITION_PULL_LEFT_DOWN:
                $objWriter->startElement('p:pull');
                $objWriter->writeAttribute('dir', 'ld');
                $objWriter->endElement();
                break;
            case Slide\Transition::TRANSITION_PULL_LEFT_UP:
                $objWriter->startElement('p:pull');
                $objWriter->writeAttribute('dir', 'lu');
                $objWriter->endElement();
                break;
            case Slide\Transition::TRANSITION_PULL_RIGHT:
                $objWriter->startElement('p:pull');
                $objWriter->writeAttribute('dir', 'r');
                $objWriter->endElement();
                break;
            case Slide\Transition::TRANSITION_PULL_RIGHT_DOWN:
                $objWriter->startElement('p:pull');
                $objWriter->writeAttribute('dir', 'rd');
                $objWriter->endElement();
                break;
            case Slide\Transition::TRANSITION_PULL_RIGHT_UP:
                $objWriter->startElement('p:pull');
                $objWriter->writeAttribute('dir', 'ru');
                $objWriter->endElement();
                break;
            case Slide\Transition::TRANSITION_PULL_UP:
                $objWriter->startElement('p:pull');
                $objWriter->writeAttribute('dir', 'u');
                $objWriter->endElement();
                break;
            case Slide\Transition::TRANSITION_PUSH_DOWN:
                $objWriter->startElement('p:push');
                $objWriter->writeAttribute('dir', 'd');
                $objWriter->endElement();
                break;
            case Slide\Transition::TRANSITION_PUSH_LEFT:
                $objWriter->startElement('p:push');
                $objWriter->writeAttribute('dir', 'l');
                $objWriter->endElement();
                break;
            case Slide\Transition::TRANSITION_PUSH_RIGHT:
                $objWriter->startElement('p:push');
                $objWriter->writeAttribute('dir', 'r');
                $objWriter->endElement();
                break;
            case Slide\Transition::TRANSITION_PUSH_UP:
                $objWriter->startElement('p:push');
                $objWriter->writeAttribute('dir', 'u');
                $objWriter->endElement();
                break;
            case Slide\Transition::TRANSITION_RANDOM:
                $objWriter->writeElement('p:random');
                break;
            case Slide\Transition::TRANSITION_RANDOMBAR_HORIZONTAL:
                $objWriter->startElement('p:randomBar');
                $objWriter->writeAttribute('dir', 'horz');
                $objWriter->endElement();
                break;
            case Slide\Transition::TRANSITION_RANDOMBAR_VERTICAL:
                $objWriter->startElement('p:randomBar');
                $objWriter->writeAttribute('dir', 'vert');
                $objWriter->endElement();
                break;
            case Slide\Transition::TRANSITION_SPLIT_IN_HORIZONTAL:
                $objWriter->startElement('p:split');
                $objWriter->writeAttribute('dir', 'in');
                $objWriter->writeAttribute('orient', 'horz');
                $objWriter->endElement();
                break;
            case Slide\Transition::TRANSITION_SPLIT_OUT_HORIZONTAL:
                $objWriter->startElement('p:split');
                $objWriter->writeAttribute('dir', 'out');
                $objWriter->writeAttribute('orient', 'horz');
                $objWriter->endElement();
                break;
            case Slide\Transition::TRANSITION_SPLIT_IN_VERTICAL:
                $objWriter->startElement('p:split');
                $objWriter->writeAttribute('dir', 'in');
                $objWriter->writeAttribute('orient', 'vert');
                $objWriter->endElement();
                break;
            case Slide\Transition::TRANSITION_SPLIT_OUT_VERTICAL:
                $objWriter->startElement('p:split');
                $objWriter->writeAttribute('dir', 'out');
                $objWriter->writeAttribute('orient', 'vert');
                $objWriter->endElement();
                break;
            case Slide\Transition::TRANSITION_STRIPS_LEFT_DOWN:
                $objWriter->startElement('p:strips');
                $objWriter->writeAttribute('dir', 'ld');
                $objWriter->endElement();
                break;
            case Slide\Transition::TRANSITION_STRIPS_LEFT_UP:
                $objWriter->startElement('p:strips');
                $objWriter->writeAttribute('dir', 'lu');
                $objWriter->endElement();
                break;
            case Slide\Transition::TRANSITION_STRIPS_RIGHT_DOWN:
                $objWriter->startElement('p:strips');
                $objWriter->writeAttribute('dir', 'rd');
                $objWriter->endElement();
                break;
            case Slide\Transition::TRANSITION_STRIPS_RIGHT_UP:
                $objWriter->startElement('p:strips');
                $objWriter->writeAttribute('dir', 'ru');
                $objWriter->endElement();
                break;
            case Slide\Transition::TRANSITION_WEDGE:
                $objWriter->writeElement('p:wedge');
                break;
            case Slide\Transition::TRANSITION_WIPE_DOWN:
                $objWriter->startElement('p:wipe');
                $objWriter->writeAttribute('dir', 'd');
                $objWriter->endElement();
                break;
            case Slide\Transition::TRANSITION_WIPE_LEFT:
                $objWriter->startElement('p:wipe');
                $objWriter->writeAttribute('dir', 'l');
                $objWriter->endElement();
                break;
            case Slide\Transition::TRANSITION_WIPE_RIGHT:
                $objWriter->startElement('p:wipe');
                $objWriter->writeAttribute('dir', 'r');
                $objWriter->endElement();
                break;
            case Slide\Transition::TRANSITION_WIPE_UP:
                $objWriter->startElement('p:wipe');
                $objWriter->writeAttribute('dir', 'u');
                $objWriter->endElement();
                break;
            case Slide\Transition::TRANSITION_ZOOM_IN:
                $objWriter->startElement('p:zoom');
                $objWriter->writeAttribute('dir', 'in');
                $objWriter->endElement();
                break;
            case Slide\Transition::TRANSITION_ZOOM_OUT:
                $objWriter->startElement('p:zoom');
                $objWriter->writeAttribute('dir', 'out');
                $objWriter->endElement();
                break;
        }

        $objWriter->endElement();
    }

    /**
     * Write Transition Slide
     * @link http://officeopenxml.com/prSlide-transitions.php
     * @param XMLWriter $objWriter
     * @param Slide\Transition $transition
     */
    protected function writeSlideTransition(XMLWriter $objWriter, $transition)
    {
        if (!$transition instanceof Slide\Transition) {
            return;
        }

        // mc:AlternateContent
        $objWriter->startElement('mc:AlternateContent');
        $objWriter->writeAttribute('xmlns:mc', 'http://schemas.openxmlformats.org/markup-compatibility/2006');
        // mc:Choice
        $objWriter->startElement('mc:Choice');
        $objWriter->writeAttribute('xmlns:p14', 'http://schemas.microsoft.com/office/powerpoint/2010/main');
        $objWriter->writeAttribute('Requires', 'p14');
        $this->writeTransition($objWriter, $transition, true);
        // mc:Choice
        $objWriter->endElement();
        // mc:Fallback
        $objWriter->startElement('mc:Fallback');
        $this->writeTransition($objWriter, $transition);
        // mc:Fallback
        $objWriter->endElement();
        // mc:AlternateContent
        $objWriter->endElement();
    }

    /**
     * @param XMLWriter $objWriter
     * @param Animation[] $oAnimations
     */
    protected function writeSlideAnimations(XMLWriter $objWriter, $oAnimations)
    {
        if (empty($oAnimations)) {
            return;
        }

        $this->cTnId = 1;
        // p:timing
        $objWriter->startElement('p:timing');
        // p:timing/p:tnLst
        $objWriter->startElement('p:tnLst');
        // p:timing/p:tnLst/p:par
        $objWriter->startElement('p:par');
        // p:timing/p:tnLst/p:par/p:cTn
        $objWriter->startElement('p:cTn');
        $objWriter->writeAttribute('id', $this->cTnId++);
        $objWriter->writeAttribute('dur', 'indefinite');
        $objWriter->writeAttribute('restart', 'never');
        $objWriter->writeAttribute('nodeType', 'tmRoot');
        // p:timing/p:tnLst/p:par/p:cTn/p:childTnLst
        $objWriter->startElement('p:childTnLst');
        // p:timing/p:tnLst/p:par/p:cTn/p:childTnLst/p:seq
        $objWriter->startElement('p:seq');
        $objWriter->writeAttribute('concurrent', '1');
        $objWriter->writeAttribute('nextAc', 'seek');
        // p:timing/p:tnLst/p:par/p:cTn/p:childTnLst/p:seq/p:cTn
        $objWriter->startElement('p:cTn');
        $objWriter->writeAttribute('id', $this->cTnId++);
        $objWriter->writeAttribute('dur', 'indefinite');
        $objWriter->writeAttribute('nodeType', 'mainSeq');
        // p:timing/p:tnLst/p:par/p:cTn/p:childTnLst/p:seq/p:cTn/p:childTnLst
        $objWriter->startElement('p:childTnLst');

        $delay = 0;
        foreach ($oAnimations as $index=>$oAnimation) {
            if ($index === 0) {
                $this->startClickPar($objWriter);
                $this->startAfterPar($objWriter, $delay);
            } else if ($oAnimation->getCondition() === Animation::CONDITION_CLICK) {
                $delay = 0;
                $this->endAfterPar($objWriter);
                $this->endClickPar($objWriter);
                $this->startClickPar($objWriter);
                $this->startAfterPar($objWriter, $delay);
            } else if ($oAnimation->getCondition() === Animation::CONDITION_AFTER) {
                $this->endAfterPar($objWriter);
                $this->startAfterPar($objWriter, $delay);
            }
            $this->writeAnimationPar($objWriter, $oAnimation);
            $delay += $oAnimation->getDuration();
        }
        $this->endAfterPar($objWriter);
        $this->endClickPar($objWriter);

        // p:timing/p:tnLst/p:par/p:cTn/p:childTnLst/p:seq/p:cTn\##p:childTnLst
        $objWriter->endElement();
        // p:timing/p:tnLst/p:par/p:cTn/p:childTnLst/p:seq\##p:cTn
        $objWriter->endElement();
        // p:timing/p:tnLst/p:par/p:cTn/p:childTnLst/p:seq/p:prevCondLst
        $objWriter->startElement('p:prevCondLst');
        // p:timing/p:tnLst/p:par/p:cTn/p:childTnLst/p:seq/p:prevCondLst/p:cond
        $objWriter->startElement('p:cond');
        $objWriter->writeAttribute('evt', 'onPrev');
        $objWriter->writeAttribute('delay', '0');
        // p:timing/p:tnLst/p:par/p:cTn/p:childTnLst/p:seq/p:prevCondLst/p:cond/p:tgtEl
        $objWriter->startElement('p:tgtEl');
        // p:timing/p:tnLst/p:par/p:cTn/p:childTnLst/p:seq/p:prevCondLst/p:cond/p:tgtEl/p:sldTgt
        $objWriter->writeElement('p:sldTgt', null);
        // p:timing/p:tnLst/p:par/p:cTn/p:childTnLst/p:seq/p:prevCondLst/p:cond\##p:tgtEl
        $objWriter->endElement();
        // p:timing/p:tnLst/p:par/p:cTn/p:childTnLst/p:seq/p:prevCondLst\##p:cond
        $objWriter->endElement();
        // p:timing/p:tnLst/p:par/p:cTn/p:childTnLst/p:seq\##p:prevCondLst
        $objWriter->endElement();
        // p:timing/p:tnLst/p:par/p:cTn/p:childTnLst/p:seq/p:nextCondLst
        $objWriter->startElement('p:nextCondLst');
        // p:timing/p:tnLst/p:par/p:cTn/p:childTnLst/p:seq/p:nextCondLst/p:cond
        $objWriter->startElement('p:cond');
        $objWriter->writeAttribute('evt', 'onNext');
        $objWriter->writeAttribute('delay', '0');
        // p:timing/p:tnLst/p:par/p:cTn/p:childTnLst/p:seq/p:nextCondLst/p:cond/p:tgtEl
        $objWriter->startElement('p:tgtEl');
        // p:timing/p:tnLst/p:par/p:cTn/p:childTnLst/p:seq/p:nextCondLst/p:cond/p:tgtEl/p:sldTgt
        $objWriter->writeElement('p:sldTgt', null);
        // p:timing/p:tnLst/p:par/p:cTn/p:childTnLst/p:seq/p:nextCondLst/p:cond\##p:tgtEl
        $objWriter->endElement();
        // p:timing/p:tnLst/p:par/p:cTn/p:childTnLst/p:seq/p:nextCondLst\##p:cond
        $objWriter->endElement();
        // p:timing/p:tnLst/p:par/p:cTn/p:childTnLst/p:seq\##p:nextCondLst
        $objWriter->endElement();
        // p:timing/p:tnLst/p:par/p:cTn/p:childTnLst\##p:seq
        $objWriter->endElement();
        // p:timing/p:tnLst/p:par/p:cTn\##p:childTnLst
        $objWriter->endElement();
        // p:timing/p:tnLst/p:par\##p:cTn
        $objWriter->endElement();
        // p:timing/p:tnLst\##p:par
        $objWriter->endElement();
        // p:timing\##p:tnLst
        $objWriter->endElement();
        // ##p:timing
        $objWriter->endElement();
    }

    protected function startGroupPar(XMLWriter $objWriter, $delay)
    {
        // p:par
        $objWriter->startElement('p:par');
        // p:par/p:cTn
        $objWriter->startElement('p:cTn');
        $objWriter->writeAttribute('id', $this->cTnId++);
        $objWriter->writeAttribute('fill', 'hold');
        // p:par/p:cTn/p:stCondLst
        $objWriter->startElement('p:stCondLst');
        // p:par/p:cTn/p:stCondLst/p:cond
        $objWriter->startElement('p:cond');
        $objWriter->writeAttribute('delay', $delay);
        // p:par/p:cTn/p:stCondLst/##p:cond
        $objWriter->endElement();
        // p:par/p:cTn/##p:stCondLst
        $objWriter->endElement();
        // p:par/p:cTn/p:childTnLst
        $objWriter->startElement('p:childTnLst');
    }

    protected function endGroupPar(XMLWriter $objWriter)
    {
        // p:par/p:cTn/##p:childTnLst
        $objWriter->endElement();
        // p:par/##p:cTn
        $objWriter->endElement();
        // ##p:par
        $objWriter->endElement();
    }

    protected function startClickPar(XMLWriter $objWriter)
    {
        $this->startGroupPar($objWriter, 'indefinite');
    }

    protected function endClickPar(XMLWriter $objWriter)
    {
        $this->endGroupPar($objWriter);
    }

    protected function startAfterPar(XMLWriter $objWriter, $delay)
    {
        $this->startGroupPar($objWriter, $delay);
    }

    protected function endAfterPar(XMLWriter $objWriter)
    {
        $this->endGroupPar($objWriter);
    }

    protected function writeAnimationPar(XMLWriter $objWriter, Animation $oAnimation)
    {
        $presetID;
        $presetClass;
        $delay = 0;
        $spid = $this->shapeIdMap[$oAnimation->getShape()->getHashCode()];
        $setParam = array(
            'spid' => $spid,
            'delay' => 0,
            'attr' => 'style.visibility',
            'val' => null
        );
        $animParam = array(
            'spid' => $spid,
            'dur' => $oAnimation->getDuration(),
            'transition' => null,
            'filter' => null
        );
        switch ($oAnimation->getType()) {
            case Animation::ANIMATION_APPEAR:
                $presetID = '1';
                $presetClass = 'entr';
                $delay = $oAnimation->getDuration();
                $setParam['val'] = 'visible';
                break;
            case Animation::ANIMATION_DISAPPEAR:
                $presetID = '1';
                $presetClass = 'exit';
                $delay = $oAnimation->getDuration();
                $setParam['val'] = 'hidden';
                break;
            case Animation::ANIMATION_FADE_IN:
                $presetID = '10';
                $presetClass = 'entr';
                $setParam['val'] = 'visible';
                $animParam['transition'] = 'in';
                $animParam['filter'] = 'fade';
                break;
            case Animation::ANIMATION_FADE_OUT:
                $presetID = '10';
                $presetClass = 'exit';
                $setParam['delay'] = max(0, $oAnimation->getDuration() - 1);
                $setParam['val'] = 'hidden';
                $animParam['transition'] = 'out';
                $animParam['filter'] = 'fade';
                break;
        }
        $nodeType;
        switch ($oAnimation->getCondition()) {
            case Animation::CONDITION_CLICK:
                $nodeType = 'clickEffect';
                break;
            case Animation::CONDITION_WITH:
                $nodeType = 'withEffect';
            break;
            case Animation::CONDITION_AFTER:
                $nodeType = 'afterEffect';
                break;
        }
        // p:par
        $objWriter->startElement('p:par');
        // p:par/p:cTn
        $objWriter->startElement('p:cTn');
        $objWriter->writeAttribute('id', $this->cTnId++);
        $objWriter->writeAttribute('presetID', $presetID);
        $objWriter->writeAttribute('presetClass', $presetClass);
        $objWriter->writeAttribute('presetSubtype', '0');
        $objWriter->writeAttribute('fill', 'hold');
        $objWriter->writeAttribute('nodeType', $nodeType);
        // p:par/p:cTn/p:stCondLst
        $objWriter->startElement('p:stCondLst');
        // p:par/p:cTn/p:stCondLst/p:cond
        $objWriter->startElement('p:cond');
        $objWriter->writeAttribute('delay', $delay);
        // p:par/p:cTn/p:stCondLst/##p:cond
        $objWriter->endElement();
        // p:par/p:cTn/##p:stCondLst
        $objWriter->endElement();
        // p:par/p:cTn/p:childTnLst
        $objWriter->startElement('p:childTnLst');

        $this->writeAnimationSet($objWriter, $setParam);
        if (!in_array($oAnimation->getType(), array(Animation::ANIMATION_APPEAR, Animation::ANIMATION_DISAPPEAR))) {
            $this->writeAnimationAnimEffect($objWriter, $animParam);
        }

        // p:par/p:cTn/##p:childTnLst
        $objWriter->endElement();
        // p:par/##p:cTn
        $objWriter->endElement();
        // ##p:par
        $objWriter->endElement();
    }

    protected function writeAnimationSet(XMLWriter $objWriter, $param)
    {
        // p:set
        $objWriter->startElement('p:set');
        // p:set/p:cBhvr
        $objWriter->startElement('p:cBhvr');
        // p:set/p:cBhvr/p:cTn
        $objWriter->startElement('p:cTn');
        $objWriter->writeAttribute('id', $this->cTnId++);
        $objWriter->writeAttribute('dur', '1');
        $objWriter->writeAttribute('fill', 'hold');
        // p:set/p:cBhvr/p:cTn/p:stCondLst
        $objWriter->startElement('p:stCondLst');
        // p:set/p:cBhvr/p:cTn/p:stCondLst/p:cond
        $objWriter->startElement('p:cond');
        $objWriter->writeAttribute('delay', $param['delay']);
        // p:set/p:cBhvr/p:cTn/p:stCondLst/##p:cond
        $objWriter->endElement();
        // p:set/p:cBhvr/p:cTn/##p:stCondLst
        $objWriter->endElement();
        // p:set/p:cBhvr/##p:cTn
        $objWriter->endElement();
        // p:set/p:cBhvr/p:tgtEl
        $objWriter->startElement('p:tgtEl');
        // p:set/p:cBhvr/p:tgtEl/p:spTgt
        $objWriter->startElement('p:spTgt');
        $objWriter->writeAttribute('spid', $param['spid']);
        // p:set/p:cBhvr/p:tgtEl/##p:spTgt
        $objWriter->endElement();
        // p:set/p:cBhvr/##p:tgtEl
        $objWriter->endElement();
        // p:set/p:cBhvr/p:attrNameLst
        $objWriter->startElement('p:attrNameLst');
        $objWriter->writeElement('p:attrName', $param['attr']);
        // p:set/p:cBhvr/##p:attrNameLst
        $objWriter->endElement();
        // p:set/##p:cBhvr
        $objWriter->endElement();
        // p:set/p:to
        $objWriter->startElement('p:to');
        // p:set/p:to/p:strVal
        $objWriter->startElement('p:strVal');
        $objWriter->writeAttribute('val', $param['val']);
        // p:set/p:to/##p:strVal
        $objWriter->endElement();
        // p:set/##p:to
        $objWriter->endElement();
        // ##p:set
        $objWriter->endElement();
    }

    protected function writeAnimationAnimEffect(XMLWriter $objWriter, $param)
    {
        // p:animEffect
        $objWriter->startElement('p:animEffect');
        $objWriter->writeAttribute('transition', $param['transition']);
        $objWriter->writeAttribute('filter', $param['filter']);
        // p:animEffect/p:cBhvr
        $objWriter->startElement('p:cBhvr');
        // p:animEffect/p:cBhvr/p:cTn
        $objWriter->startElement('p:cTn');
        $objWriter->writeAttribute('id', $this->cTnId++);
        $objWriter->writeAttribute('dur', $param['dur']);
        // p:animEffect/p:cBhvr/##p:cTn
        $objWriter->endElement();
        // p:animEffect/p:cBhvr/p:tgtEl
        $objWriter->startElement('p:tgtEl');
        // p:animEffect/p:cBhvr/p:tgtEl/p:spTgt
        $objWriter->startElement('p:spTgt');
        $objWriter->writeAttribute('spid', $param['spid']);
        // p:animEffect/p:cBhvr/p:tgtEl/##p:spTgt
        $objWriter->endElement();
        // p:animEffect/p:cBhvr/##p:tgtEl
        $objWriter->endElement();
        // p:animEffect/##p:cBhvr
        $objWriter->endElement();
        // ##p:animEffect
        $objWriter->endElement();
    }

    private function getGUID()
    {
        if (function_exists('com_create_guid')) {
            return com_create_guid();
        } else {
            mt_srand((double)microtime() * 10000);//optional for php 4.2.0 and up.
            $charid = strtoupper(md5(uniqid(rand(), true)));
            $hyphen = chr(45);// "-"
            $uuid = chr(123)// "{"
                . substr($charid, 0, 8) . $hyphen
                . substr($charid, 8, 4) . $hyphen
                . substr($charid, 12, 4) . $hyphen
                . substr($charid, 16, 4) . $hyphen
                . substr($charid, 20, 12)
                . chr(125);// "}"
            return $uuid;
        }
    }

    private function writeCNvPr(XMLWriter $objWriter, $shape)
    {
        $shapeId = $this->shapeId++;
        $this->shapeIdMap[$shape->getHashCode()] = $shapeId;

        $objWriter->startElement('p:cNvPr');
        $objWriter->writeAttribute('id', $shapeId);
        if ($shape->isPlaceholder()) {
            $objWriter->writeAttribute('name', 'Placeholder for ' . $shape->getPlaceholder()->getType());
        } else {
            $name = '';
            if ($shape instanceof Group) {
                $name = 'Group ' . $shapeId;
            } else if (property_exists($shape, 'getName')) {
                $name = $shape->getName();
            }
            $objWriter->writeAttribute('name', $name);
        }
        if (property_exists($shape, 'getDescription')) {
            $objWriter->writeAttribute('descr', $shape->getDescription());
        }
        $this->writeHyperlink($objWriter, $shape);
        $objWriter->endElement();

    }

    private function writeNvPr(XMLWriter $objWriter, $shape)
    {
        $objWriter->startElement('p:nvPr');
        if ($shape->isPlaceholder()) {
            $objWriter->startElement('p:ph');
            $objWriter->writeAttribute('type', $shape->getPlaceholder()->getType());
            if (!is_null($shape->getPlaceholder()->getIdx())) {
                $objWriter->writeAttribute('idx', $shape->getPlaceholder()->getIdx());
            }
            $objWriter->endElement();
        }
        if ($shape instanceof Media) {
            $objWriter->startElement('a:videoFile');
            $objWriter->writeAttribute('r:link', $shape->relationId);
            $objWriter->endElement();

            $objWriter->startElement('p:extLst');
            $objWriter->startElement('p:ext');
            $objWriter->writeAttribute('uri', '{DAA4B4D4-6D71-4841-9C94-3DE7FCFB9230}');
            $objWriter->startElement('p14:media');
            $objWriter->writeAttribute('r:embed', $shape->relationId);
            $objWriter->writeAttribute('xmlns:p14', 'http://schemas.microsoft.com/office/powerpoint/2010/main');
            $objWriter->endElement();
            $objWriter->endElement();
            $objWriter->endElement();
        }
        $objWriter->endElement();
    }

    private function writeTransform(XMLWriter $objWriter, $shape)
    {
        if (is_null($shape->getOffsetX()) && is_null($shape->getOffsetY()) &&
            is_null($shape->getWidth()) && is_null($shape->getHeight())) {
            return;
        }

        if ($shape instanceof ShapeTable || $shape instanceof ShapeChart) {
            $tag = 'p:xfrm';
        } else {
            $tag = 'a:xfrm';
        }

        $objWriter->startElement($tag);
        $objWriter->writeAttributeIf($shape->getRotation() != 0, 'rot', CommonDrawing::degreesToAngle($shape->getRotation()));
        $objWriter->writeAttributeIf($shape->getFlipH(), 'flipH', '1');
        $objWriter->writeAttributeIf($shape->getFlipV(), 'flipV', '1');

        if (!is_null($shape->getOffsetX()) && !is_null($shape->getOffsetY())) {
            $objWriter->startElement('a:off');
            $objWriter->writeAttribute('x', CommonDrawing::pixelsToEmu($shape->getOffsetX()));
            $objWriter->writeAttribute('y', CommonDrawing::pixelsToEmu($shape->getOffsetY()));
            $objWriter->endElement();
        }

        if (!is_null($shape->getWidth()) && !is_null($shape->getHeight())) {
            $objWriter->startElement('a:ext');
            $objWriter->writeAttribute('cx', CommonDrawing::pixelsToEmu($shape->getWidth()));
            $objWriter->writeAttribute('cy', CommonDrawing::pixelsToEmu($shape->getHeight()));
            $objWriter->endElement();
        }

        if ($shape instanceof Group) {
            $objWriter->startElement('a:chOff');
            $objWriter->writeAttribute('x', CommonDrawing::pixelsToEmu($shape->getOffsetX()));
            $objWriter->writeAttribute('y', CommonDrawing::pixelsToEmu($shape->getOffsetY()));
            $objWriter->endElement();

            $objWriter->startElement('a:chExt');
            $objWriter->writeAttribute('cx', CommonDrawing::pixelsToEmu($shape->getWidth()));
            $objWriter->writeAttribute('cy', CommonDrawing::pixelsToEmu($shape->getHeight()));
            $objWriter->endElement();
        }

        $objWriter->endElement();
    }
}
