<?php
/**
 * This file is part of PHPWord - A pure PHP library for reading and writing
 * word processing documents.
 *
 * PHPWord is free software distributed under the terms of the GNU Lesser
 * General Public License version 3 as published by the Free Software Foundation.
 *
 * For the full copyright and license information, please read the LICENSE
 * file that was distributed with this source code. For the full list of
 * contributors, visit https://github.com/PHPOffice/PHPWord/contributors.
 *
 * @link        https://github.com/PHPOffice/PHPWord
 * @copyright   2010-2014 PHPWord contributors
 * @license     http://www.gnu.org/licenses/lgpl.txt LGPL version 3
 */

namespace PhpOffice\PhpWord\Reader\Word2007;

use PhpOffice\PhpWord\PhpWord;
use PhpOffice\PhpWord\Shared\XMLReader;

/**
 * Styles reader
 *
 * @since 0.10.0
 */
class Styles extends AbstractPart
{
    /**
     * Read styles.xml.
     *
     * @param \PhpOffice\PhpWord\PhpWord $phpWord
     * @return void
     */
    public function read(PhpWord $phpWord)
    {
        $xmlReader = new XMLReader();
        $xmlReader->getDomFromZip($this->docFile, $this->xmlFile);

        if ($xmlReader->elementExists('w:docDefaults/w:rPrDefault/w:rPr')) {
            $node = $xmlReader->getElement('w:docDefaults/w:rPrDefault');
            $defaultStyle = $this->readFontStyle($xmlReader, $node);
            $phpWord->setDefaultFontStyle($defaultStyle);
        }

        if ($xmlReader->elementExists('w:docDefaults/w:pPrDefault/w:pPr')) {
            $node = $xmlReader->getElement('w:docDefaults/w:pPrDefault');
            $defaultStyle = $this->readParagraphStyle($xmlReader, $node);
            $phpWord->setDefaultParagraphStyle($defaultStyle);
        }

        $nodes = $xmlReader->getElements('w:style');
        if ($nodes->length > 0) {
            foreach ($nodes as $node) {
                $type = $xmlReader->getAttribute('w:type', $node);
                $name = $xmlReader->getAttribute('w:styleId', $node);
                $alt_name = $xmlReader->getAttribute('w:val', $node, 'w:name');
                $basedOn = $xmlReader->getAttribute('w:val', $node, 'w:basedOn');
                if (is_null($name)) {
                    $name = $alt_name;
                }
                preg_match('/Heading(\d)/', $name, $headingMatches);
                preg_match('/heading\s*(\d)+/i', $alt_name, $altHeadingMatches);
                // $default = ($xmlReader->getAttribute('w:default', $node) == 1);
                switch ($type) {

                    case 'paragraph':
                        $paragraphStyle = $this->readParagraphStyle($xmlReader, $node);
                        $paragraphStyle['basedOn'] = $basedOn;
                        $fontStyle = $this->readFontStyle($xmlReader, $node);

                        if (empty($paragraphStyle['outlineLevel'])) {
                            if (!empty($headingMatches)) {
                                $paragraphStyle['outlineLevel'] = intval($headingMatches[1] - 1);
                            } else if (!empty($altHeadingMatches)) {
                                $paragraphStyle['outlineLevel'] = intval($altHeadingMatches[1] - 1);
                            }
                        }
                        if (empty($fontStyle)) {
                            if (is_array($paragraphStyle)) {
                                $phpWord->addParagraphStyle($name, $paragraphStyle);
                            }
                        } else {
                            $fontStyle['basedOn'] = $basedOn;
                            $phpWord->addFontStyle($name, $fontStyle, $paragraphStyle);
                        }
                        break;

                    case 'character':
                        $fontStyle = $this->readFontStyle($xmlReader, $node);
                        if ( empty($fontStyle) ) {
                            $fontStyle = array();
                        }
                        $fontStyle['basedOn'] = $basedOn;
                        $phpWord->addFontStyle($name, $fontStyle);
                        break;

                    case 'table':
                        $tStyle = $this->readTableStyle($xmlReader, $node);
                        if ( empty($tStyle) ) {
                            $tStyle = array();
                        }
                        $tStyle['basedOn'] = $basedOn;
                        $phpWord->addTableStyle($name, $tStyle);
                        break;

                    case 'numbering':
                        \PhpOffice\PhpWord\Style::setNumStyleLink($name, 'PHPWordListMSWord' . $xmlReader->getAttribute(
                                'w:val', $node, 'w:pPr/w:numPr/w:numId'));
                        break;
                }
            }
        }
    }
}
