<?php
/**
 * This file is part of PHPWord - A pure PHP library for reading and writing
 * word processing documents.
 *
 * PHPWord is free software distributed under the terms of the GNU Lesser
 * General Public License version 3 as published by the Free Software Foundation.
 *
 * For the full copyright and license information, please read the LICENSE
 * file that was distributed with this source code. For the full list of
 * contributors, visit https://github.com/PHPOffice/PHPWord/contributors.
 *
 * @link        https://github.com/PHPOffice/PHPWord
 * @copyright   2010-2014 PHPWord contributors
 * @license     http://www.gnu.org/licenses/lgpl.txt LGPL version 3
 */

namespace PhpOffice\PhpWord\Writer\HTML\Element;

/**
 * Table element HTML writer
 *
 * @since 0.10.0
 */
class Table extends AbstractElement
{
    /**
     * Write table
     *
     * @return string
     */
    public function write()
    {
        if (!$this->element instanceof \PhpOffice\PhpWord\Element\Table) {
            return '';
        }

        // Build table grid
        $grid = array();
        $rows = $this->element->getRows();
        for ($i = 0, $numRows = count($rows); $i < $numRows; ++$i) {
            $index = 0;
            $row = $rows[$i];
            $grid[] = array(
                'element' => $row,
                'height' => $row->getHeight(),
                'cells' => array());
            $cells = $row->getCells();
            for ($j = 0, $numCells = count($cells); $j < $numCells; ++$j) {
                $cell = $cells[$j];
                $cellStyle = $cell->getStyle();

                if ($cellStyle->getVMerge() === 'continue') {
                    $k = $i - 1;
                    while (($k >= 0) && !$grid[$k]['cells'][$index]['real']) {
                        --$k;
                    }
                    if ($k < 0) {
                        // TODO
                        return '';
                    }
                    for ($l = 0, $colspan = $grid[$k]['cells'][$index]['colspan']; $l < $colspan; ++$l) {
                        ++$grid[$k]['cells'][$index]['rowspan'];
                        $grid[$i]['cells'][$index] = array(
                            'element' => $cell,
                            'width' => $cell->getWidth(),
                            'rowspan' => $grid[$k]['cells'][$index]['rowspan'],
                            'colspan' => $grid[$k]['cells'][$index]['colspan'],
                            'real' => false);
                        ++$index;
                    }
                    continue;
                }

                $colspan = $cellStyle->getGridSpan();
                if (!$colspan) {
                    $colspan = 1;
                }
                $colspan = (int)$colspan;
                for ($k = 0; $k < $colspan; ++$k) {
                    $grid[$i]['cells'][$index] = array(
                        'element' => $cell,
                        'width' => $cell->getWidth(),
                        'rowspan' => 1,
                        'colspan' => $colspan,
                        'real' => ($k === 0));
                    ++$index;
                }
            }
        }

        if (count($grid) < 1) {
            return '';
        }

        // Write grid
        $content = '<table>' . PHP_EOL;
        for ($i = 0, $numRows = count($grid); $i < $numRows; ++$i) {
            $row = $grid[$i];
            $content .= '<tr';
            if ($row['height']) {
                $height = (int)$row['height'] / 20;
                $content .= " style=\"height:{$height}px;\"";
            }
            $content .= '>' . PHP_EOL;
            for ($j = 0, $numCells = count($row['cells']); $j < $numCells; ++$j) {
                $cell = $row['cells'][$j];
                if (!$cell['real']) {
                    continue;
                }
                $writer = new Container($this->parentWriter, $cell['element']);
                $cellTag = $tblHeader ? 'th' : 'td';
                $content .= "<{$cellTag} rowspan=\"{$cell['rowspan']}\" colspan=\"{$cell['colspan']}\"";
                if ($cell['width']) {
                    $width = (int)$cell['width'] / 20;
                    $content .= " style=\"width:{$width}px;\"";
                }
                $content .= '>' . PHP_EOL;
                $content .= $writer->write();
                $content .= "</{$cellTag}>" . PHP_EOL;
            }
            $content .= '</tr>' . PHP_EOL;
        }
        $content .= '</table>' . PHP_EOL;

        return $content;
    }
}
